package Mentat::Reporter::Cache::Default;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Reporter::Cache::Default

=head1 SYNOPSIS

 use Mentat::Reporter::Cache::Default;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), Mentat::Reporter::Cache(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Mentat::Reporter::Cache;
use Value::Convertor;
use Data::Filter::Factory;
use Mentat::Storage::Mongo;
use Mentat::Message::Convertor::Mongo;
use Mentat::Message::Storage::Mongo;
use Mentat::Message::Query::Mongo;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Reporter::Cache');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item check_threshold_cache($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on instance

=cut

sub check_threshold_cache($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($keys, $tsl) = @_;

    my $rv;
    foreach my $key (@$keys) {
        # Return the in memory cached value, if possible
        if (exists $self->{T_CACHE_GET}->{$key}) {
            #Log::Loger->info(__PACKAGE__, "[ENG:Reporter:Cache] Threshold memory cache hit for '$key':'$tsl' = '$self->{T_CACHE_GET}->{$key}'");
            $rv = (defined($rv)) ? (($self->{T_CACHE_GET}->{$key} and $rv)?1:0) : ($self->{T_CACHE_GET}->{$key});
        }
        else {

            my $count = $self->{STORAGE}->count({'_id' => $key, 'ts' => {'$gte' => int($tsl)}});
            if ($count) {
                Log::Loger->info(__PACKAGE__, "[ENG:Reporter:Cache] Threshold storage cache hit for '$key':'$tsl' (".Value::Convertor->unixs_to_ldatestr($tsl).")");
                $self->{T_CACHE_GET}->{$key} = 1;
                $rv = (defined($rv)) ? (($self->{T_CACHE_GET}->{$key} and $rv)?1:0) : ($self->{T_CACHE_GET}->{$key});
            }
            else {
                Log::Loger->info(__PACKAGE__, "[ENG:Reporter:Cache] Threshold storage cache miss for '$key':'$tsl' (".Value::Convertor->unixs_to_ldatestr($tsl).")");
                $self->{T_CACHE_GET}->{$key} = 0;
                $rv = (defined($rv)) ? (($self->{T_CACHE_GET}->{$key} and $rv)?1:0) : ($self->{T_CACHE_GET}->{$key});
            }

        }
    }

    return $rv;
}

=item set_threshold_cache($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on instance

=cut

sub set_threshold_cache($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($keys, $ts) = @_;

    foreach my $key (@$keys) {
        unless (exists $self->{T_CACHE_SET}->{$key}) {
            Log::Loger->info(__PACKAGE__, "[ENG:Reporter:Cache] Threshold storage cache insert for '$key':'$ts' (".Value::Convertor->unixs_to_ldatestr($ts).")");
            eval {
                $self->{STORAGE}->update({ '_id' => $key }, { '_id' => $key, 'ts' => int($ts) }, {'upsert' => 1});
            };
            if ($@) {
                Log::Loger->error(__PACKAGE__, "[ENG:Reporter:Cache] Error setting threshold cache: '$@'");
            }
            $self->{T_CACHE_GET}->{$key} = 1;
            $self->{T_CACHE_SET}->{$key} = 1;
        }
        else {
            Log::Loger->info(__PACKAGE__, "[ENG:Reporter:Cache] Threshold storage cache record already present for '$key':'$ts' (".Value::Convertor->unixs_to_ldatestr($ts).")");
        }
    }
}

=item check_relapse_cache($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on instance

=cut

sub check_relapse_cache($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($keys, $tsl) = @_;

    my $rv;
    foreach my $key (@$keys) {
        # Return the in memory cached value, if possible
        if (exists $self->{R_CACHE_GET}->{$key}) {
            #Log::Loger->info(__PACKAGE__, "[ENG:Reporter:Cache] Relapse memory cache hit for '$key':'$tsl' = '$self->{R_CACHE_GET}->{$key}'");
            $rv = (defined($rv)) ? (($self->{R_CACHE_GET}->{$key} and $rv)?1:0) : ($self->{R_CACHE_GET}->{$key});
        }
        else {

            my $count = $self->{STORAGE}->count({'_id' => $key, 'ts' => {'$lte' => int($tsl)}});
            if ($count) {
                #Log::Loger->info(__PACKAGE__, "[ENG:Reporter:Cache] Relapse storage cache hit for '$key':'$tsl' (".Value::Convertor->unixs_to_ldatestr($tsl).")");
                $self->{R_CACHE_GET}->{$key} = 1;
                $rv = (defined($rv)) ? (($self->{R_CACHE_GET}->{$key} and $rv)?1:0) : ($self->{R_CACHE_GET}->{$key});
            }
            else {
                #Log::Loger->info(__PACKAGE__, "[ENG:Reporter:Cache] Relapse storage cache miss for '$key':'$tsl' (".Value::Convertor->unixs_to_ldatestr($tsl).")");
                $self->{R_CACHE_GET}->{$key} = 0;
                $rv = (defined($rv)) ? (($self->{R_CACHE_GET}->{$key} and $rv)?1:0) : ($self->{R_CACHE_GET}->{$key});
            }

        }
    }

    return $rv;
}

=item set_relapse_cache($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on instance

=cut

sub set_relapse_cache($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($keys, $ts) = @_;

    foreach my $key (@$keys) {
        unless (exists $self->{R_CACHE_SET}->{$key}) {
            #Log::Loger->info(__PACKAGE__, "[ENG:Reporter:Cache] Relapse storage cache delete for '$key':'$ts' (".Value::Convertor->unixs_to_ldatestr($ts).")");
            $self->{STORAGE}->remove({'_id' => $key, 'ts' => {'$lte' => int($ts)}});
            $self->{R_CACHE_SET}->{$key} = 1;
            $self->{R_CACHE_GET}->{$key} = 0;
            $self->{T_CACHE_GET}->{$key} = 0;
        }
        else {
            #Log::Loger->info(__PACKAGE__, "[ENG:Reporter:Cache] Relapse storage cache record already removed for '$key':'$ts' (".Value::Convertor->unixs_to_ldatestr($ts).")");
        }
    }
}

=item cleanup(;$$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on instance

=cut

sub cleanup(;$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($threshold, $ts) = @_;

    $threshold = 0    unless defined $threshold and int($threshold);
    $ts        = time unless defined $ts and int($ts);

    $ts -= $threshold;

    Log::Loger->notice(__PACKAGE__, "[ENG:Reporter:Cache] Cleaning up cache with threshold '$ts' (".Value::Convertor->unixs_to_ldatestr($ts).")");
    my $rv = $self->{STORAGE}->remove({'ts' => {'$lte' => int($ts)}});
    if ($rv->{ok}) {
        Log::Loger->notice(__PACKAGE__, "[ENG:Reporter:Cache] Cleanup successfull, removed '".$rv->{n}."' items from threshold cache");
    }
    else {
        Log::Loger->notice(__PACKAGE__, "[ENG:Reporter:Cache] Cleanup failure: '".$rv->{err}."'");
    }
    return $rv;
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Reporter::Cache::Default instance
#  Arguments : Unknown see the description in appropriate subclass
#  Returns   : Mentat::Reporter::Cache::Default $self reference
#  Throws    : Dies, if not invoked on instance or given invalid arguments
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Initializing Mentat::Reporter::Cache::Default instance...
    my %config = @_;

    my $db  = ($config{threshold_cache_db})  ? $config{threshold_cache_db}  : 'mentat';
    my $col = ($config{threshold_cache_col}) ? $config{threshold_cache_col} : 'cache_threshold';
    # Setup different default cache for test data mode
    if ($config{test_data}) {
        $db  = 'mentat'               unless $config{threshold_cache_db};
        $col = 'test_cache_threshold' unless $config{threshold_cache_col};
    }

    # Database cache storage
    $self->{STORAGE} = Mentat::Storage::Mongo->new(database => $db, collection => $col);
    # Memory cache storage
    $self->{CACHE}   = {};

    Log::Loger->info(__PACKAGE__, "[ENG:Reporter:Cache] Using '$db:$col' as threshold cache");

    return $self;
}

=pod

=back

=cut

1;
