package Mentat::Stats::Brief;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Stats::Brief - Short module description

=head1 SYNOPSIS

 use Mentat::Stats::Brief;

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), Mentat::Stats(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Time::Piece;

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Mentat::Stats;
use Mentat::Stats::Data;
use Mentat::Storage::Mongo;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant SECONDS_IN_DAY => 86400;

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Stats');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item DESTROY() [PUBLIC]

  Usage     : This method must not be used directly
  Purpose   : Class destructor
  Arguments : None
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance

=cut

sub DESTROY()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Destroying Mentat::Stats::Brief instance...
}

#-------------------------------------------------------------------------------

sub brief_date($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($time,) = @_;

    my $t = gmtime($time);
    return $t->ymd;
}

sub calculate_thresholds($$;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($date, $type, $previous) = @_;

    my %TYPES = (
        'day'   => 1,
        'week'  => 1,
        'month' => 1,
        'year'  => 1,
    );

    unless ($date) {
        my $t = gmtime;
        $date = $t->ymd;
    }
    unless ($type) {
        $type = 'day';
    }
    confess "Invalid type '$type' for brief threshold calculation" unless $TYPES{$type};

    my ($th, $tl);
    my $time = Time::Piece->strptime($date, "%Y-%m-%d");

    if ($type eq 'day') {
        $tl = $time->epoch;
        $th = $tl + SECONDS_IN_DAY();
    }
    elsif ($type eq 'week') {
        $tl = $time->epoch;
        $th = $tl + (7 * SECONDS_IN_DAY());
    }
    else {
        my ($l, $h);
        if ($type eq "month") {
            $l = "".sprintf("%4d", $time->year)."-".sprintf("%02d", $time->mon)."-".sprintf("%02d", 1);
            $h = ($time->mon == 12)
                    ? "".sprintf("%4d", ($time->year+1))."-".sprintf("%02d", 1)             ."-".sprintf("%02d", 1)
                    : "".sprintf("%4d", ($time->year))  ."-".sprintf("%02d", ($time->mon+1))."-".sprintf("%02d", 1);
        }
        elsif ($type eq "year") {
            $l = "".sprintf("%4d", $time->year)    ."-".sprintf("%02d", 1)."-".sprintf("%02d", 1);
            $h = "".sprintf("%4d", ($time->year+1))."-".sprintf("%02d", 1)."-".sprintf("%02d", 1);
        }
        my $tsl = Time::Piece->strptime($l, "%Y-%m-%d");
        my $tsh = Time::Piece->strptime($h, "%Y-%m-%d");
        $tl = $tsl->epoch;
        $th = $tsh->epoch;
    }

    return ($th, $tl, $date, $type);
}

=item calc_stats_processing($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   : HASH REFERENCE $stats
  Throws    : Dies, if not invoked on instance

=cut

sub calc_stats_processing($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($th, $tl) = @_;

    my $storage = Mentat::Storage::Mongo->new(database => 'mentat_stats', collection => 'statistics');

    my ($objects, $count) = $storage->find({'ts_to' => {'$lte' => $th}, 'ts_from' => {'$gte' => $tl}}, undef, {'ts_from' => 1});

    my $sengine = Mentat::Stats::Data->new(charts_dir => $self->{CHARTS_DIR});

    return $sengine->ng_evaluate_alert_stats($objects);
}

=item calc_stats_reporting($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   : HASH REFERENCE $stats
  Throws    : Dies, if not invoked on instance

=cut

sub calc_stats_reporting($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($th, $tl) = @_;

    my $storage = Mentat::Storage::Mongo->new(database => 'mentat', collection => 'reports_ng');

    my ($objects, $count) = $storage->find({'ts' => {'$exists' => 1}, 'test_data' => 0, 'ts_to' => {'$lte' => $th}, 'ts_from' => {'$gte' => $tl}}, undef, {'ts_from' => 1});

    my $sengine = Mentat::Stats::Data->new(charts_dir => $self->{CHARTS_DIR});

    return $sengine->ng_evaluate_reports($objects, 0, 1);
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Stats::Brief instance
#  Arguments : Unknown see the description in appropriate subclass
#  Returns   : Mentat::Stats::Brief $self reference
#  Throws    : Dies, if not invoked on instance or given invalid arguments
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Initializing Mentat::Stats::Brief instance...
    my %config = @_;

    die "[BRF] Chart directory '$config{'charts_dir'}' does not exist\n" unless -d $config{charts_dir};
    die "[BRF] Chart directory '$config{'charts_dir'}' is not writable\n" unless -w $config{charts_dir};

    $self->{CHARTS_DIR} = $config{charts_dir};

    if ($self->{CHARTS_DIR} =~ /(.*)/) { $self->{CHARTS_DIR} = $1; }

    return $self;
}

=pod

=back

=cut

1;
